"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.componentExecOptions = exports.componentCompose = exports.componentRestart = exports.componentDestroy = exports.componentStop = exports.componentStart = exports.componentIsRunning = exports.componentExec = exports.componentExecCompose = exports.initComponent = void 0;
const colors_1 = require("kleur/colors");
const node_tty_1 = __importDefault(require("node:tty"));
const componentConfig_1 = require("./componentConfig");
const core_1 = require("./core");
const initComponent = async (component) => {
    const ctx = new Map(component.workspace.context);
    ctx.set('APP_NAME', component.name);
    ctx.set('COMPOSE_PROJECT_NAME', `${component.workspace.config?.name}-${component.name}`);
    ctx.set('COMPOSE_PROJECT_NAME', `${component.workspace.config?.name}-${component.name}`);
    const svcPath = (0, core_1.substVars)(component.config.path, ctx);
    ctx.set('SVC_PATH', svcPath);
    if (component.config.extends) {
        const template = component.workspace.config.components.get(component.config.extends);
        if (!template)
            throw new Error(`Error extending component ${component.name} from ${component.config.extends}: ${component.config.extends} is not defined. Known components: ${[...component.workspace.config.components.keys()].join(', ')}`);
        component.template = template;
        const templatePath = (0, core_1.substVars)(template.path, ctx);
        ctx.set('TPL_PATH', templatePath);
        if (!template.composeFile) {
            template.composeFile = '${TPL_PATH}/docker-compose.yml';
        }
        const composeFile = (0, core_1.substVars)(template.composeFile, ctx);
        ctx.set('COMPOSE_FILE', composeFile);
        for (const [key, v] of Object.entries(template.variables)) {
            const value = (0, core_1.substVars)(v, ctx);
            ctx.set(key, value);
        }
    }
    if (component.config.composeFile) {
        const composeFile = (0, core_1.substVars)(component.config.composeFile, ctx);
        ctx.set('COMPOSE_FILE', composeFile);
    }
    if (!component.config.composeFile && !ctx.has('COMPOSE_FILE')) {
        const composeFile = (0, core_1.substVars)(`\${SVC_PATH}/docker-compose.yml`, ctx);
        ctx.set('COMPOSE_FILE', composeFile);
    }
    const vars = typeof component.config?.variables === 'object' ? Object.entries(component.config.variables) : [];
    for (const [key, val] of vars) {
        const value = (0, core_1.substVars)(val, ctx);
        ctx.set(key, value);
    }
    component.context = ctx;
};
exports.initComponent = initComponent;
const componentExecCompose = async (comp, composeCommand, options, interactive = false) => {
    const composeFile = comp.context.get('COMPOSE_FILE');
    const command = ['docker', 'compose', '-f', composeFile, ...composeCommand];
    const commandStr = command.join(' ');
    if (options.debug) {
        console.log(`>> ${interactive ? '(interactive)' : ''}${commandStr}`);
    }
    if (options.dryRun)
        return '';
    const env = {};
    for (const [k, v] of comp.context.entries()) {
        env[k] = v;
    }
    if (options.debug) {
        console.log('>> ENV:', env);
    }
    if (interactive) {
        await (0, core_1.execShellInteractive)(commandStr, comp.workspace.cwd, env);
        return '';
    }
    return (0, core_1.execShellToString)(commandStr, comp.workspace.cwd, env);
};
exports.componentExecCompose = componentExecCompose;
const componentExec = async (comp, command, options, interactive = false) => {
    const commandStr = command.join(' ');
    if (options.debug) {
        console.log(`>> ${commandStr}`);
    }
    if (options.dryRun)
        return '';
    const env = {};
    for (const [k, v] of comp.context.entries()) {
        env[k] = v;
    }
    if (interactive) {
        await (0, core_1.execShellInteractive)(commandStr, comp.workspace.cwd, env);
        return '';
    }
    return (0, core_1.execShellToString)(commandStr, comp.workspace.cwd, env);
};
exports.componentExec = componentExec;
const componentIsRunning = async (comp, options) => {
    const out = await (0, exports.componentExecCompose)(comp, ['ps', '--status=running', '-q'], options);
    return out !== '';
};
exports.componentIsRunning = componentIsRunning;
const componentStart = async (comp, options) => {
    const running = await (0, exports.componentIsRunning)(comp, options);
    console.log('starting component: ', comp.name, 'is it running?', running);
    if (!running || options.force) {
        const deps = (0, componentConfig_1.getDeps)(comp.config, options.mode || 'default');
        const comps = (0, componentConfig_1.resolveDeps)(comp.workspace, deps);
        console.log((0, colors_1.red)('   deps: ' + comps.map(e => e.name).join(',')));
        for (const comp of comps) {
            const running = await (0, exports.componentIsRunning)(comp, options);
            if (!running || options.force) {
                await (0, exports.componentExecCompose)(comp, ['up', '-d'], options, true);
            }
        }
    }
    if (!running) {
        return (0, exports.componentExecCompose)(comp, ['up', '-d'], options, true);
    }
    else {
        console.log((0, colors_1.bold)((0, colors_1.green)(`🚀 Component "${comp.name}" is already running`)));
    }
};
exports.componentStart = componentStart;
const componentStop = async (comp, options) => {
    const running = await (0, exports.componentIsRunning)(comp, options);
    if (!running)
        return;
    await (0, exports.componentExecCompose)(comp, ['stop'], options, true);
};
exports.componentStop = componentStop;
const componentDestroy = async (comp, options) => {
    const running = await (0, exports.componentIsRunning)(comp, options);
    if (!running)
        return;
    return (0, exports.componentExecCompose)(comp, ['down'], options, true);
};
exports.componentDestroy = componentDestroy;
const componentRestart = async (comp, hard, options) => {
    if (hard) {
        await (0, exports.componentDestroy)(comp, options);
    }
    else {
        await (0, exports.componentStop)(comp, options);
    }
    return (0, exports.componentStart)(comp, {});
};
exports.componentRestart = componentRestart;
const componentCompose = async (comp, options) => {
    return (0, exports.componentExecCompose)(comp, options.cmd, options, true);
};
exports.componentCompose = componentCompose;
const componentExecOptions = async (comp, options) => {
    await (0, exports.componentStart)(comp, options);
    const command = ['exec'];
    if (options.workingDir) {
        command.push(...['-w', options.workingDir]);
    }
    if (typeof options.UID === 'number') {
        command.push(...['-u', `${options.UID}`]);
    }
    else {
        const userId = comp.context.get('USER_ID');
        if (!userId)
            throw new Error('variable "USER_ID" is not set');
        const groupId = comp.context.get('GROUP_ID');
        if (!groupId)
            throw new Error('variable "GROUP_ID" is not set');
        command.push(...['-u', `${userId}:${groupId}`]);
    }
    if (options.noTty || !node_tty_1.default.isatty(process.stdout.fd)) {
        command.push('-T');
    }
    command.push('app');
    command.push(...options.cmd);
    return (0, exports.componentExecCompose)(comp, command, options, true);
};
exports.componentExecOptions = componentExecOptions;
