"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.resolveDeps = exports.getDeps = exports.mergeComponentConfigs = void 0;
const mergeComponentConfigs = (cc, cc2) => {
    if (cc2.replace)
        return { ...cc2 };
    const result = { ...cc };
    const replacable = [
        'path',
        'composeFile',
        'extends',
        'hostedIn',
        'execPath',
        'alias',
        'repository',
        'afterCloneHook',
    ];
    for (const key of replacable) {
        if (cc2[key])
            result[key] = cc2[key];
    }
    for (const [k, v] of result.variables) {
        result.variables.set(k, v);
    }
    result.tags = [...(result.tags || []), ...(cc2.tags || [])];
    if (cc2.dependencies) {
        for (const [depSvc, modes] of Object.entries(cc2.dependencies)) {
            if (!result.dependencies.has(depSvc)) {
                result.dependencies.set(depSvc, new Array());
            }
            for (const mode of modes) {
                if (!result.dependencies.get(depSvc)?.includes(mode)) {
                    result.dependencies.get(depSvc)?.push(mode);
                }
            }
        }
    }
    return result;
};
exports.mergeComponentConfigs = mergeComponentConfigs;
const getDeps = (cc, mode) => {
    const result = [];
    for (const [key, modes] of Object.entries(cc.dependencies)) {
        if (modes.includes(mode)) {
            result.push(key);
        }
    }
    return result;
};
exports.getDeps = getDeps;
const resolveDeps = (workspace, deps) => {
    let components = [];
    for (const dep of deps) {
        const comp = workspace.components.get(dep);
        if (!comp)
            throw new Error(`Could not find component "${dep}"`);
        components.push(comp);
        if (comp.config.dependencies &&
            typeof comp.config.dependencies === 'object' &&
            Object.entries(comp.config.dependencies).length > 0) {
            components = [...(0, exports.resolveDeps)(workspace, Object.keys(comp.config.dependencies)), ...components];
        }
    }
    return components;
};
exports.resolveDeps = resolveDeps;
