"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GeneratorRequires = exports.getDependencyOrder = exports.dependencies = void 0;
exports.dependencies = new Map();
const getDependencyOrder = () => {
    const graph = new Map();
    const depKeys = exports.dependencies.keys();
    // 1. Find root
    for (const dependant of depKeys) {
        const deps = exports.dependencies.get(dependant);
        for (const dep of deps) {
            if (!graph.has(dep)) {
                graph.set(dep, {
                    name: dep,
                    in: [],
                    out: [],
                });
            }
            graph.get(dep).out.push(dependant);
        }
        if (!graph.has(dependant)) {
            graph.set(dependant, {
                in: [],
                out: [],
                name: dependant,
            });
        }
        graph.get(dependant).in.push(...deps);
    }
    let root = undefined;
    for (const [k, v] of graph.entries()) {
        if (v.in.length === 0) {
            if (typeof root !== 'undefined')
                throw new Error(`Error: multiple root dependencies. Please check these generators: [${[root, k].join()}]`);
            root = k;
        }
    }
    if (!root)
        throw new Error(`Error: no root dependency found`);
    // 2. Determine execution steps
    const result = [];
    const q = [graph.get(root)];
    while (q.length) {
        const cur = q.shift();
        const hasMoreInputsThanLoaded = cur.in.length && cur.in.some(e => !result.includes(e));
        if (!hasMoreInputsThanLoaded && !result.includes(cur.name)) {
            result.push(cur.name);
        }
        if (cur.out.length) {
            q.push(...cur.out.map(e => graph.get(e)));
        }
    }
    return result;
};
exports.getDependencyOrder = getDependencyOrder;
const GeneratorRequires = (others) => {
    return (target) => {
        if (!exports.dependencies.has(target.name)) {
            exports.dependencies.set(target.name, []);
        }
        exports.dependencies.get(target.name).push(...others.map(e => e.name));
    };
};
exports.GeneratorRequires = GeneratorRequires;
