"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.generators = void 0;
const fs_1 = require("fs");
const colors_1 = require("kleur/colors");
const prompts_1 = __importDefault(require("prompts"));
const yargs_parser_1 = __importDefault(require("yargs-parser"));
const package_json_1 = require("../package.json");
const context_1 = require("./context");
const decorators_1 = require("./decorators");
const Api_1 = __importDefault(require("./generators/Api"));
const GitHooks_1 = __importDefault(require("./generators/GitHooks"));
const Kafka_1 = __importDefault(require("./generators/Kafka"));
const LintFormat_1 = __importDefault(require("./generators/LintFormat"));
const Prisma_1 = __importDefault(require("./generators/Prisma"));
const Scaffold_1 = __importDefault(require("./generators/Scaffold"));
exports.generators = [
    Prisma_1.default,
    Api_1.default,
    GitHooks_1.default,
    LintFormat_1.default,
    Scaffold_1.default,
    Kafka_1.default,
];
const disclaimer = `
Welcome to the Uniform setup wizard!
There are definitely bugs and some features might not work yet.
If you encounter an issue, have a look at https://github.com/iamcsharper/Uniform/issues and open a new one, if it is not already tracked.
`;
async function main() {
    console.log((0, colors_1.gray)(`\ncreate-uniform-app version ${package_json_1.version}`));
    console.log((0, colors_1.red)(disclaimer));
    const context = await (0, context_1.loadContext)();
    console.log('Context=', context);
    const args = (0, yargs_parser_1.default)(context.yargs);
    const targetFolder = args[0] ||
        (await (0, prompts_1.default)({
            type: 'text',
            name: 'value',
            message: 'Where do you want to create',
            initial: 'my-app',
        })).value;
    const isExists = (0, fs_1.existsSync)(targetFolder);
    if (isExists) {
    }
    const plugins = (await (0, prompts_1.default)({
        type: 'multiselect',
        name: 'plugins',
        message: 'Which plugins do you want to use?',
        choices: [...decorators_1.dependencies.keys()].map(generator => ({
            title: generator.replace('Generator', ''),
            value: generator,
        })),
        initial: 0,
    })).plugins;
    if (!plugins.length) {
        throw new Error('No plugins selected');
    }
    context.plugins = plugins;
    context.targetFolder = targetFolder;
    const namedGenerators = new Map();
    for (const generator of exports.generators) {
        namedGenerators.set(generator.name, generator);
    }
    const ordered = (0, decorators_1.getDependencyOrder)();
    const orderedFiltered = ordered.filter(e => plugins.includes(e) || e === 'ScaffoldGenerator');
    for (const generatorName of orderedFiltered) {
        const Constructor = namedGenerators.get(generatorName);
        const instance = new Constructor();
        await instance.generate(context);
    }
    console.log((0, colors_1.green)('✅ Successfully generated'));
}
exports.default = main;
main().catch(console.error);
