"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.initWorkspace = exports.loadWorkspace = exports.createWorkspaceContext = exports.createWorkspace = void 0;
const fs_1 = require("fs");
const path_1 = require("path");
const component_1 = require("./component");
const core_1 = require("./core");
const workspaceConfig_1 = require("./workspaceConfig");
const createWorkspace = (wsPath, cwd) => ({
    aliases: new Map(),
    components: new Map(),
    cwd,
    configPath: wsPath,
    context: new Map(),
});
exports.createWorkspace = createWorkspace;
const createWorkspaceContext = (ws) => {
    const ctx = new Map();
    ctx.set('WORKSPACE_PATH', ws.configPath);
    ctx.set('WORKSPACE_NAME', ws.config.name || '');
    for (const pair of ws.config.variables) {
        const value = (0, core_1.substVars)(pair[1], ctx);
        ctx.set(pair[0], value);
    }
    return ctx;
};
exports.createWorkspaceContext = createWorkspaceContext;
const loadWorkspace = async (ws) => {
    let wsc = await (0, workspaceConfig_1.loadWorkspaceConfig)((0, path_1.join)(ws.configPath, 'uniform.json'));
    const envPath = (0, path_1.join)(ws.configPath, 'env.json');
    if ((0, fs_1.existsSync)(envPath)) {
        const envWsc = await (0, workspaceConfig_1.loadWorkspaceConfig)(envPath);
        wsc = (0, workspaceConfig_1.mergeWorkspaceConfig)(wsc, envWsc);
    }
    ws.config = wsc;
};
exports.loadWorkspace = loadWorkspace;
const initWorkspace = async (ws) => {
    const ctx = (0, exports.createWorkspaceContext)(ws);
    ws.context = ctx;
    for (const [compName, compCfg] of ws.config.components) {
        ws.components.set(compName, {
            name: compName,
            config: {
                tags: [],
                variables: new Map(),
                dependencies: new Map(),
                ...compCfg,
            },
            workspace: ws,
        });
    }
    for (const [_, comp] of ws.components) {
        await (0, component_1.initComponent)(comp);
    }
};
exports.initWorkspace = initWorkspace;
