"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.loadWorkspaceConfig = exports.mergeWorkspaceConfig = void 0;
const promises_1 = require("fs/promises");
const componentConfig_1 = require("./componentConfig");
const mergeWorkspaceConfig = (wsc, wsc2) => {
    const result = { ...wsc };
    for (const [name, cc] of wsc2.components) {
        if (!result.components.has(name)) {
            result.components.set(name, cc);
        }
        else {
            result.components.set(name, (0, componentConfig_1.mergeComponentConfigs)(result.components.get(name), cc));
        }
    }
    for (const [k, v] of wsc2.variables) {
        result.variables.set(k, v);
    }
    return result;
};
exports.mergeWorkspaceConfig = mergeWorkspaceConfig;
const loadWorkspaceConfig = async (wscPath) => {
    const file = await (0, promises_1.readFile)(wscPath, 'utf-8');
    const data = JSON.parse(file);
    const result = {
        components: new Map(),
        variables: new Map(),
        name: data.name || undefined,
    };
    if (typeof data.components === 'object') {
        for (const [key, value] of Object.entries(data.components)) {
            result.components.set(key, value);
        }
    }
    if (typeof data.variables === 'object') {
        for (const [key, value] of Object.entries(data.variables)) {
            result.variables.set(key, value);
        }
    }
    return result;
};
exports.loadWorkspaceConfig = loadWorkspaceConfig;
